//---------------------------------------------------------------------------
#ifndef FSNotifyH
#define FSNotifyH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
//---------------------------------------------------------------------------
namespace       Fsnotify
{

  enum TNotifyFilter {
    cFileName, cDirName, cAttributes, cSize, cLastWrite, cSecurity
  };

  typedef         Set < TNotifyFilter, cFileName, cSecurity > TNotifyFilters;

  class           Notify
  {
    private:
    HANDLE FHandle;
    bool            FResult;
    DWORD           FTimeOut;
                    public:

                    Notify(const String & Path, bool WSubTree, TNotifyFilters Filters, DWORD TimeOut);
                   ~Notify() {
      FindCloseChangeNotification(FHandle);
    }
    bool            Next();

  };

  // class TThNotifyWalker;
  typedef void    __fastcall(__closure * TThNotifyWalkerError)
                  (System::TObject * Sender, int ErrNum, const String & Message);

  enum TNWError {
    eDirExist, eContinue
  };

  class           TThNotifyWalker:public TThread
  {
    private:
    DWORD FTimeOut;
    bool            FSync;
    String          FPath;
    bool            FFirst;
    bool            FWSubTree;
    TNWError        FError;
    TNotifyFilters  FFilters;
    String          FOptMsg;

                    protected:

                    TNotifyEvent FOnEvent;

    TThNotifyWalkerError FOnError;
    void __fastcall Error();

    void __fastcall SendEvent() {
      if (FOnEvent)
	FOnEvent(this);
    }
    virtual void __fastcall Execute(void);

  public:
    __fastcall virtual TThNotifyWalker();
    __fastcall virtual ~ TThNotifyWalker() {}
  public:
    __property TNotifyEvent OnChange = {read = FOnEvent, write = FOnEvent};
    __property TThNotifyWalkerError OnError = {read = FOnError, write = FOnError};
    __property OnTerminate;
    __property String Path = {write = FPath};
    __property DWORD TimeOut = {write = FTimeOut,read = FTimeOut};
    __property bool Sync = {write = FSync};
    __property bool First = {write = FFirst};
    __property bool WSubTree = {write = FWSubTree};
    __property TNotifyFilters Filters = {write = FFilters};
    __property Terminated;

  };

  class PACKAGE   TFSNotify:public TComponent
  {
   private:
    TThNotifyWalker * FWlk;
    TNotifyFilters  FFilters;
    bool            FActive;
    String          FPath;
    TNotifyEvent    FOnEvent;
    TThNotifyWalkerError FOnError;
    DWORD           FTimeOut;
    bool            FFirst;
    bool            FSubTree;

  protected:
    void __fastcall SetFilters(TNotifyFilters Filters);
    void __fastcall SetActive(bool Active);
    void __fastcall SetPath(String Path);

  public:
    __fastcall virtual TFSNotify(TComponent * Owner);
    __fastcall virtual ~ TFSNotify();
  __published:
    __property TNotifyFilters Filters = {read = FFilters, write = SetFilters ,default = cFileName};
    __property bool Active = {read = FActive, write = SetActive, default = false};
    __property String Path = {read = FPath, write = SetPath };
    __property TNotifyEvent OnChange = {read = FOnEvent, write = FOnEvent};
    __property TThNotifyWalkerError OnError = {read = FOnError, write = FOnError};
    __property DWORD TimeOut = {read = FTimeOut, write = FTimeOut, default = INFINITE};
    __property bool First = {read = FFirst, write = FFirst , default = false};
    __property bool SubTree = {read = FSubTree, write = FSubTree , default = false };
  };

  //---------------------------------------------------------------------------
  //-- var, const, procedure ---------------------------------------------------
  extern PACKAGE void __fastcall Register(void);

}

#if !defined(NO_IMPLICIT_NAMESPACE_USE)
using namespace Fsnotify;
#endif
//-- end .h ----------------------------------------------------------------
#endif // namespace Fsnotify

